'''performance
'''

_measure = 50 # a +50 point grade difference predicts 1-0
_limit = 40 # so a player always gets a +reward for winning
_span = _measure * 2
_frame = max([_span * 2 + 3, _span + 3]) # minimum _span + 3
_split = _frame / 2 # pick out high and low halves of gap... lists
_win = 1
_draw = 0.5
_loss = 0


class Performance(object):
    '''Grades calculated from a collection of results.
    '''
    

    def __init__(self, events=None):

        self.performances = {}
        self.points = {}
        self.games = {}
        self.grades = {}
        self.scores = {}
        self.predictedscores = {}
        self.gapgames = []
        self.gapscores = []
        self.gappredictedscores = []
        if events == None:
            self.events = None
            self.players = {}
        else:
            self.events = events
            self.players = events.GetPlayers()
            for p in self.players:
                self.performances[p] = [0]
                self.scores[p] = 0
                self.predictedscores[p] = 0
            self.gapgames.extend([0] * _frame)
            self.gapscores.extend([0] * _frame)
            self.gappredictedscores.extend([0] * _frame)
    

    def DoIterations(self,
                     count=10,
                     calculation=None,
                     initialgrade=None,
                     finalcalculation=None):

        if initialgrade == None:
            initialgrade = {}
        for p in self.players:
            self.performances[p] = [initialgrade.get(p, 0)]
        for i in range(count):
            self.IterateNewGrade(calculation)
        if callable(finalcalculation):
            self.IterateNewGrade(finalcalculation)
        self.events.ProcessAllResults(self.TotalScore)
        self.events.ProcessAllResults(self.TotalGapScore)
        perf = self.performances
        g = self.grades
        for p in perf:
            del perf[p][1:-3]
            g[p] = int(round(perf[p][-1]))


    def GradeDifference(self, player, opponent, score):

        points = self.performances[opponent][-1]
        self.IncrementPoints(player, points, score)
    

    def GradeDifference40(self, player, opponent, score):

        p = self.performances[player][-1]
        o = self.performances[opponent][-1]
        if p - o > _limit:
            points = p - _limit
        elif o - p > _limit:
            points = p + _limit
        else:
            points = o
        self.IncrementPoints(player, points, score)
    

    def IncrementPoints(self, player, points, score):

        if score == '1':
            points += _measure
        elif score == '0':
            points -= _measure
        self.points[player] += points
        self.games[player] += 1
    

    def IterateNewGrade(self, calculation=None):

        if not callable(calculation):
            calculation = self.GradeDifference

        points = self.points
        games = self.games

        for p in self.players:
            points[p] = 0
            games[p] = 0
            
        self.events.ProcessAllResults(calculation)

        for player in games.keys():
            if games[player] == 0:
                del games[player]
                del points[player]

        for player in points:
            self.performances[player].append(float(points[player]) / games[player])


    def Reward(self, player, opponent, score):

        gap = self.performances[player][-1] - self.performances[opponent][-1]
        if gap > _measure:
            expected = _win
        elif -gap > _measure:
            expected = _loss
        else:
            expected = (gap + _measure) / _span
        if score == '1':
            actual = _win
        elif score == '5':
            actual = _draw
        else:
            actual = _loss
        return (actual, expected, gap)
    

    def TotalGapScore(self, player, opponent, score):

        actual, expected, gap = self.Reward(player, opponent, score)
        slot = int(round(gap))
        if slot > _span:
            self.gappredictedscores[-1] += expected
            self.gapscores[-1] += actual
            self.gapgames[-1] += 1
        elif -slot > _span:
            self.gappredictedscores[0] += expected
            self.gapscores[0] += actual
            self.gapgames[0] += 1
        else:
            slot += _span + 1
            self.gappredictedscores[slot] += expected
            self.gapscores[slot] += actual
            self.gapgames[slot] += 1
    

    def TotalScore(self, player, opponent, score):

        actual, expected, gap = self.Reward(player, opponent, score)
        self.predictedscores[player] += expected
        self.scores[player] += actual
    

if __name__ == '__main__':

    from os.path import abspath
    from ECFresults import ResultFolder

    rf1 = ResultFolder(abspath('results/Results 2005-2006'))
    rf2 = ResultFolder(abspath('results/Results 2004-2005'))
    
    a = Performance(rf1)
    b = Performance(rf1)
    c = Performance(rf2)
    
    a.DoIterations(50)
    b.DoIterations(50, b.GradeDifference40)
    c.DoIterations(50)
    
    d = Performance(rf1)
    e = Performance(rf1)
    f = Performance(rf1)
    g = Performance(rf1)
    
    d.DoIterations(50, initialgrade = c.grades)
    e.DoIterations(100, initialgrade = c.grades)
    f.DoIterations(5)
    g.DoIterations(50, finalcalculation = g.GradeDifference40)
    
    a0 = a.players
    a1 = a.grades
    a2 = a.performances
    a3 = a.scores
    a4 = a.predictedscores
    b1 = b.grades
    d1 = d.grades
    e1 = e.grades
    f1 = f.grades
    g1 = g.grades
    for n in a0:
        print a0[n], a1[n], a3[n], a4[n], b1[n], d1[n], e1[n], f1[n], g1[n], a2[n]

    gg = a.gapgames
    gs = a.gapscores
    gp = a.gappredictedscores
    for n in range(_frame):
        print n, gg[n], gs[n], gp[n]
